from huibiao_framework.utils.meta_class import OsAttrMeta, ConstantClass


class MinioConfig(metaclass=OsAttrMeta):
    ENDPOINT: str
    AK: str
    SK: str
    BUCKET_NAME: str = "huibiao"
    OSS_SECURE: bool = False


class LlmModelNameConstant(ConstantClass):
    HuiZeQwen32bQwq = "HuiZeQwen32bQwq"

class ModelConfig(metaclass=OsAttrMeta):
    """大语言模型的参数"""
    LLM_MODEL_TYPE: str = "HuiZeQwen32bQwq"
    REQUEST_URL: str = "http://vllm-qwen-32b.model.hhht.ctcdn.cn:9080/common/query"


class ClientConfig:
    SEARCH_URL: str = "http://knowledge-qa-search-pipeline-mgr-svc-dev.ctcdn.cn:9080"
    IMAGE_OCR_TYY_URL: str = (
        "http://tender-document-parser.hhht.ctcdn.cn:9080/image_ocr"
    )
    LAYOUT_DETECTION_TYY_URL: str = (
        "http://tender-document-parser.hhht.ctcdn.cn:9080/image_layout"
    )
    EMBED_URL: str = "http://dmx.model.zz.ctcdn.cn:9080/vllm-huize-embedding-zhengwu0329/common/encode"
    CHUNK_URL: str = (
        "http://dmx.model.dev.zz.ctcdn.cn:9080/text-content-chunk-tools-dev/chunk"
    )



from enum import Enum


class BasicStatusCode(Enum):
    def __new__(cls, value: int, msg: str):
        obj = object.__new__(cls)
        obj._value_ = obj
        obj._code = value
        obj._msg = msg
        return obj

    @property
    def value(self):
        return self._code

    @property
    def msg(self):
        return self._msg


class CommonStatusCode(BasicStatusCode):
    SUCCESS = (0, "成功")

    ERROR_REQ_URL = (40001, "请求路径错误")
    ERROR_REQ_METHOD = (40002, "请求方法错误")
    BODY_EMPTY_ERR = (40003, "请求体内容为空")
    PARAM_ERROR = (40006, "参数错误:{msg}")
    ERROR_HEADER_NOW_EXISTS = (40007, "缺少请求头:{header}")

    UNKNOWN_ERROR = (50000, "未知错误: {msg}")

    INTERNAL_ERROR = (50001, "内部错误: {msg}")



from huibiao_framework.backend.status_code import BasicStatusCode


class BasicException(Exception):
    """
    基础异常类
    """

    def __init__(self, code: int, msg: str):
        super().__init__(msg)
        self.code = code
        self.msg = msg


class BasicCommonException(BasicException):
    def __init__(self, status_code: BasicStatusCode, **kwargs):
        super().__init__(code=status_code.value, msg=status_code.msg.format(**kwargs))



from .status_code import BasicStatusCode, CommonStatusCode
from .exception import BasicException, BasicCommonException
from .vo import BaseRespVo


__all__ = [
    "BasicStatusCode",
    "CommonStatusCode",
    "BasicException",
    "BasicCommonException",
    "BaseRespVo",
]



from pydantic import BaseModel, Field
from typing import List, Tuple, Literal


class LayoutRegionVo(BaseModel):
    """文档布局中单个区域的信息模型"""

    # 识别置信度分数，范围0-1
    score: float = Field(..., ge=0.0, le=1.0, description="区域识别的置信度分数")

    # 区域的四边形坐标，由4个(x,y)坐标点组成
    poly: List[Tuple[int, int]] = Field(
        ...,
        min_items=4,
        max_items=4,
        description="区域的四边形边界坐标，包含4个(x,y)顶点",
    )

    # 区域类型，限制为预定义的几种类型
    type: Literal["header", "title", "paragraph", "seal", "table"] = Field(
        ..., description="区域的语义类型"
    )


class DocumentLayoutDetectVo(BaseModel):
    """分析结果的详细信息模型"""

    # 文档宽度（像素）
    width: int = Field(..., gt=0, description="文档宽度，单位为像素")

    # 文档高度（像素）
    height: int = Field(..., gt=0, description="文档高度，单位为像素")

    # 文档旋转角度
    angle: int = Field(..., description="文档旋转角度，单位为度")

    # 分析模型版本
    version: str = Field(..., description="分析使用的模型版本")

    # 布局区域列表
    layouts: List[LayoutRegionVo] = Field(..., description="文档中识别到的布局区域列表")



import functools
import os

import aiofiles
import json

import aiohttp
from loguru import logger

from huibiao_framework.client import MinIOClient
from huibiao_framework.config import MinioConfig


def with_oss_client(func):
    @functools.wraps(func)
    async def wrapper(self, client: MinIOClient = None, *args, **kwargs):
        self.valid_oss_param()

        if client:
            return await func(self, client, *args, **kwargs)
        else:
            async with aiohttp.ClientSession() as session:
                client = MinIOClient(session=session)
                return await func(self, client, *args, **kwargs)
    return wrapper


class JsonFile:
    def __init__(
        self,
        local_path: str,
        wget_link: str = None,
        oss_key=None,
        oss_bucket: str = MinioConfig.BUCKET_NAME,
    ):
        self.local_path = local_path
        self.wget_link = wget_link
        self.oss_bucket = oss_bucket
        self.oss_key = oss_key
        self.__data = None

        os.makedirs(os.path.dirname(self.local_path), exist_ok=True)

    @property
    def data(self):
        return self.__data

    def set_data(self, data) -> 'JsonFile':
        self.__data = data
        return self

    def exists(self) -> bool:
        return os.path.exists(self.local_path)

    async def load_json(self):
        """
        异步io加载
        """
        async with aiofiles.open(self.local_path, "r", encoding="utf-8") as f:
            self.__data = json.loads(await f.read())
            logger.debug(f"load json {self.local_path}")
            return self.__data

    async def save_json(self):
        """
        异步io保存
        """
        async with aiofiles.open(self.local_path, "w", encoding="utf-8") as f:
            await f.write(json.dumps(self.__data, ensure_ascii=False, indent=4))
            logger.debug(f"save json {self.local_path}")

    async def wget(self, session: aiohttp.ClientSession = None):
        """下载json，保存到本地"""
        assert self.wget_link, "请设置json文件的下载链接"

        async def fun(_session):
            async with _session.get(self.wget_link) as response:
                response.raise_for_status()
                data = json.loads(await response.read())
                logger.info(f"wget json {self.wget_link} -> {self.local_path}")
                self.__data = data
                await self.save_json()

        if session is not None:
            await fun(session)
        else:
            async with aiohttp.ClientSession() as session:
                await fun(session)

    def valid_oss_param(self):
        assert self.oss_key, "请设置json文件oss_key"
        assert self.oss_bucket, "请设置json文件oss_bucket"

    @with_oss_client
    async def upload_oss(self, client: MinIOClient = None):
        """上传到oss"""
        self.valid_oss_param()
        await client.upload_file(
            bucket_name=self.oss_bucket,
            object_name=self.oss_key,
            file_path=self.local_path,
        )

    @with_oss_client
    async def download_oss(self, client: MinIOClient = None):
        """从oss下载到本地"""
        self.valid_oss_param()
        await client.download_file(
            bucket_name=self.oss_bucket,
            object_name=self.oss_key,
            file_path=self.local_path,
        )
        await self.load_json()

    @with_oss_client
    async def gen_oss_url(self, client: MinIOClient = None):
        """生成oss链接"""
        self.valid_oss_param()
        return await client.get_object_url(
            bucket_name=self.oss_bucket,
            object_name=self.oss_key,
        )


from abc import ABC, abstractmethod


class AbstractLlmModelClient(ABC):
    @abstractmethod
    async def simple_query(self, content, session_id: str = "") -> str:
        """
        简单的一次性问答
        LLMException
        """
        pass



import sys

from loguru import logger
import os


class LoguruUtils:
    @classmethod
    def service_setup(
        cls,
        service_name: str,
        log_dir: str,
        *,
        add_pid_suffix: bool = True,
        max_size_mb: int = 100,
        remove_exist: bool = True,
    ):
        """
        日志输出终端和落盘
        """
        if remove_exist:
            logger.remove()  # 清空设置，防止重复

        os.makedirs(log_dir, exist_ok=True)
        pid_suffix = f"_{os.getpid()}" if add_pid_suffix else ""

        # 添加终端处理器（控制台输出）
        logger.add(
            sink=sys.stderr,  # 输出到标准错误流
            level="DEBUG",  # 终端显示更详细的DEBUG日志
            format="{time:YYYY-MM-DD HH:mm:ss.SSS} | {level: <8} | {name}:{function}:{line} - {message}",
            colorize=True,  # 启用颜色显示
            backtrace=True,  # 堆栈信息显示在终端
        )

        # 配置 INFO 及以上级别日志
        logger.add(
            os.path.join(log_dir, f"{service_name}_info{pid_suffix}.log"),
            rotation=f"{max_size_mb} MB",
            filter=lambda record: record["level"].no >= 20,
            format="{time:YYYY-MM-DD HH:mm:ss.SSS} | {level: <8} | {process} | {message}",
            enqueue=True,
        )

        # 配置 DEBUG 级别日志
        logger.add(
            os.path.join(log_dir, f"{service_name}_debug{pid_suffix}.log"),
            rotation=f"{max_size_mb} MB",
            level="DEBUG",
            filter=lambda record: record["level"].no >= 10,
            format="{time:YYYY-MM-DD HH:mm:ss.SSS} | {level: <8} | {name}:{function}:{line} - {message}",
            enqueue=True,
        )

        logger.info("日志落盘设置完成")



from typing import List, Optional, Dict, Any
from pydantic import BaseModel


class ChunkReqDto(BaseModel):
    """
    请求参数示例
    {
        "Action": "CommonChunk",
        "TextData": "ddddddddddddd",
        "FileName": "ddddddd",
        "IsJson": false
    }
    """

    Action: str = "CommonChunk"
    TextData: str
    FileName: str
    IsJson: bool


class ChunkRespVo(BaseModel):
    """
    返回值示例
    {
    "code": 0,
    "result": {
        "docInfo": {
            "docId": 3714767230599517013,
            "docName": "ddddddd"
        },
        "contents": [
            {
                "info": {
                    "contentId": 1,
                    "docTitle": "ddddddd",
                    "subtitle": "",
                    "text": "ddddddddddddd",
                    "offset": 13,
                    "strategy": "ARTICLE_SPLITTER"
                },
                "chunks": [
                    {
                        "id": 1,
                        "text": "ddddddd\nddddddddddddd",
                        "type": "HEAD_SENTENCE_FEATURE",
                        "offset": 21
                    }
                ]
            }
        ],
        "outline": {
            "outlineStr": "",
            "offset": 0,
            "items": [],
            "type": "RE_OUTLINE"
        },
        "redHeader": {
            "红头": "",
            "发文号": "",
            "发文单位": "",
            "发文日期": "",
            "发文类型": "",
            "发文数量": ""
        }
    },
    "message": "success"
    }
    """

    docInfo: Optional["DocInfo"]
    contents: Optional[List["ContentItem"]]
    outline: Optional["Outline"]
    redHeader: Optional[Dict[str, Any]]


class DocInfo(BaseModel):
    docId: int
    docName: str


class Chunk(BaseModel):
    id: int
    text: str
    type: str
    offset: int


class ContentInfo(BaseModel):
    contentId: int
    docTitle: str
    subtitle: str
    text: str
    offset: int
    strategy: str


class ContentItem(BaseModel):
    info: ContentInfo
    chunks: List[Chunk]


class Outline(BaseModel):
    outlineStr: str
    offset: int
    items: List
    type: str



import asyncio
import time
from collections.abc import Coroutine
from typing import List, Optional

from loguru import logger


async def limit_batch_gather(task: List[Coroutine], *, batch: int = 4, log_tag: Optional[str] = ""):
    """
    将task列表按照batch尺寸拆开，串行执行每个batch
    返回结果的顺序和输入任务的顺序对应
    使用示例
    async def mytask(idx: int):
        await asyncio.sleep(1)
        logger.info(f"mytask done {idx}")
        return idx

    async def main():
        all_result = await limit_batch_gather([mytask(idx) for idx in range(34)], batch=5, log_tag=None)
        logger.info(all_result)

    asyncio.run(main())
    """
    log_tag = log_tag + " " if log_tag else ""

    batch_list = [task[i : i + batch] for i in range(0, len(task), batch)]
    batch_result = []
    start_time = time.perf_counter()
    logger.debug(f"{log_tag}Batch task start, total {len(task)} task")
    for idx, b_task in enumerate(batch_list):
        b_start_time = time.perf_counter()
        batch_result.append(await asyncio.gather(*b_task))
        logger.debug(f"{log_tag}[{idx + 1}/{len(batch_list)}]-[{len(b_task)}] Batch task done, "
                     f"cost={time.perf_counter() - b_start_time:.6f}s")

    logger.debug(f"{log_tag}All batch task done, cost={time.perf_counter() - start_time:.6f}")
    # 结果合并，保留原顺序
    result = [item for sublist in batch_result for item in sublist]

    return result